/**
* DevExtreme (ui/resizable.d.ts)
* Version: 19.2.9 (build 20226-1038)
* Build date: Thu Aug 13 2020
*
* Copyright (c) 2012 - 2020 Developer Express Inc. ALL RIGHTS RESERVED
* Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
*/
import '../jquery_augmentation';

import DOMComponent, {
    DOMComponentOptions
} from '../core/dom_component';

import {
    dxElement
} from '../core/element';

import {
    event
} from '../events';

/** Warning! This type is used for internal purposes. Do not import it directly. */
export interface dxResizableOptions extends DOMComponentOptions<dxResizable> {
    /** Specifies which borders of the widget element are used as a handle. */
    handles?: 'bottom' | 'left' | 'right' | 'top' | 'all' | string;
    /** Specifies the widget's height. */
    height?: number | string | (() => number | string);
    /** Specifies the upper height boundary for resizing. */
    maxHeight?: number;
    /** Specifies the upper width boundary for resizing. */
    maxWidth?: number;
    /** Specifies the lower height boundary for resizing. */
    minHeight?: number;
    /** Specifies the lower width boundary for resizing. */
    minWidth?: number;
    /** A function that is executed each time the widget is resized by one pixel. */
    onResize?: ((e: { component?: dxResizable, element?: dxElement, model?: any, jQueryEvent?: JQueryEventObject, event?: event, width?: number, height?: number }) => any);
    /** A function that is executed when resizing ends. */
    onResizeEnd?: ((e: { component?: dxResizable, element?: dxElement, model?: any, jQueryEvent?: JQueryEventObject, event?: event, width?: number, height?: number }) => any);
    /** A function that is executed when resizing starts. */
    onResizeStart?: ((e: { component?: dxResizable, element?: dxElement, model?: any, jQueryEvent?: JQueryEventObject, event?: event, width?: number, height?: number }) => any);
    /** Specifies the widget's width. */
    width?: number | string | (() => number | string);
}
/** The Resizable widget enables its content to be resizable in the UI. */
export default class dxResizable extends DOMComponent {
    constructor(element: Element, options?: dxResizableOptions)
    constructor(element: JQuery, options?: dxResizableOptions)
}

declare global {
interface JQuery {
    dxResizable(): JQuery;
    dxResizable(options: "instance"): dxResizable;
    dxResizable(options: string): any;
    dxResizable(options: string, ...params: any[]): any;
    dxResizable(options: dxResizableOptions): JQuery;
}
}
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type Options = dxResizableOptions;

/** @deprecated use Options instead */
/** Warning! This type is used for internal purposes. Do not import it directly. */
export type IOptions = dxResizableOptions;